--[[
FantasyLoot
Item link modification

Author: Michael Fu "Mithorium" (Morodan @ Khadgar-US)

The code of this addon is licensed under a Creative Commons Attribution-Noncommercial-Share Alike 3.0 License.
(see license.txt) All included textures and sounds are copyrighted by their respective owners, license information
for these media files can be found in the modules that make use of them.

 You are free:
   * to Share - to copy, distribute, display, and perform the work
   * to Remix - to make derivative works
 Under the following conditions:
   * Attribution.   You must attribute the work in the manner specified by the author or licensor (but not in any way
                    that suggests that they endorse you or your use of the work).
   * Noncommercial. You may not use this work for commercial purposes.
   * Share Alike.   If you alter, transform, or build upon this work, you may distribute the resulting work only under
                    the same or similar license to this one.
]]
local _, _AG = ...
local L = _AG.L

-- /* Setup */
FantasyLoot = LibStub("AceAddon-3.0"):NewAddon("FantasyLoot", "AceConsole-3.0")

-- /* Locals */
_AG.upgradeByType = _AG.upgradeByType or {
  [1] = "Valor t16", [2] = "Mystery t16", [3] = "Valor t15", [4] = "Mystery t15",
  ["Valor t16"] = {levels = 4, [1] = {start = 491, levels = 2}, [2] = {start = 504, levels = 1}},
  ["Mystery t16"] = {levels = 6, [1] = {start = 494, levels = 4}, [2] = {start = 506, levels = 1}},
  ["Valor t15"] = {start = 465, levels = 2},
  ["Mystery t15"] = {start = 468, levels = 4},
}
local upgradeByType = _AG.upgradeByType

_AG.gemIndex = _AG.gemIndex or {
  "Uncommon", "Rare"
}
local gemIndex = _AG.gemIndex

_AG.modifierTable = _AG.modifierTable or {
  [1] = "Warforged", [2] = "Heroic", [3] = "Mythic", [4] = "Raid Finder",
  ["Warforged"] = 448,
  ["Heroic"] = 449,
  ["Mythic"] = 450,
  ["Raid Finder"] = 451
}
local modifierTable = _AG.modifierTable

-- Default DB
FantasyLoot.defaultDB = {
  db_version = 1,
  -- savedEnchants = {
    -- [4442] = "Jade Spirit",
    -- [4444] = "Dancing Steel",
  -- },
  gemsKnown = 0,
  gemToStatsByType = {
    Uncommon    = {},
    Rare   = {},
  },
}

local ItemMeta = {}

-- Helpers

local function packageItemLink(i)
  -- mystery is 0 or 6
  -- modifiercount 2-16, max 3
  local modifiers = ""
  if i.modifiercount > 0 then
    for m = 1, i.modifiercount do
      modifiers = modifiers..":"..i["modifier"..tostring(m)]
    end
  end
  return "|cff"..i.color.."|Hitem:"..i.id..":"..i.enchant..":"..i.gem1..":"..i.gem2..":"..i.gem3..":"..i.gem4..":"..i.suffix..":"..i.unique..":"..i.linklvl..":"..i.upgrade..":"..i.mystery..":"..i.modifiercount..modifiers.."|h["..i.name.."]|h|r"
end

local function println(text)
  DEFAULT_CHAT_FRAME:AddMessage(("|cffff7d0a[|r|cffffd200FantasyLoot|r|cffff7d0a]|r %s"):format(tostring(text)), 0.9, 1, 0.8)
end

local function printerr(text)
  DEFAULT_CHAT_FRAME:AddMessage(("|cffff7d0a[|r|cffffd200FantasyLoot|r|cffff7d0a]|r %s"):format(tostring(text)), 1, 0.2, 0.25)
end

local function InitDB(tbl, defaults)
  if type(tbl) == "string" then
    local name = tbl
    tbl = _G[name]
    if not tbl then
      tbl = defaults
      _G[name] = tbl
    end
  end

  return tbl
end

-- /* Init */
function FantasyLoot:OnInitialize()
  -- Create DB
  self.db = InitDB("FantasyLootDB", self.defaultDB)
  if not self.db then
    printerr("Error: Database corrupted. Please remove FantasyLoot.lua from your saved variables folder.")
  end
  
  self:ClearItem()
  
  self.frame = nil
  
  self:RegisterChatCommand("fantasyloot", "OnCommand")
  self:RegisterChatCommand("fl", "OnCommand")
  
  FantasyLootUI:OnInitialize()
end

-- /* API */
function FantasyLoot:OnCommand(argv)
  arg = self:GetArgs(argv) or ""
  
  if arg == "toggle" then
    if FantasyLootFrame:IsShown() then
      self:Hide()
    else
      self:Show()
    end
  elseif arg == "hide" then
    self:Hide()
  elseif tonumber(arg) ~= nil then
    self:SetItemID(arg)
    self:Show()
  else
    self:SetItem(arg)
    self:Show()
  end
end

function FantasyLoot:Show()
  if not FantasyLootFrame then
    return
  end
  if not FantasyLootFrame:IsShown() then
    FantasyLootFrame:Show()
  else
    FantasyLootUI:Update(FantasyLootFrame)
  end
end

function FantasyLoot:Hide()
  if not FantasyLootFrame then
    return
  end
  if FantasyLootFrame:IsShown() then
    FantasyLootFrame:Hide()
  end
end

function FantasyLoot:SetItemID(itemID)
  local itemLink = select(2, GetItemInfo(itemID))
  
  if itemLink == nil then
    return false
  end
  
  return self:SetItem(itemLink)
end

function FantasyLoot:SetItem(itemLink)
  local color, id, enchant, gem1, gem2, gem3, gem4, suffix, unique, linklvl, upgrade, mystery, modifiercount, modifiers, name =
  itemLink:match("|cff(%x+)|Hitem:(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%-?%d+):(%d+):(%-?%d+):(%d+):(%d+):(%d+)([^|]*)|h%[([^%]]+)%]|h|r")
  
  if id == nil then
    return false
  end
  
  modifiercount = tonumber(modifiercount)
  if modifiercount > 3 then
    modifiercount = 3
  end
  
  local i = self.item
  i.color = color
  i.id = id
  i.enchant = enchant
  i.gem1 = gem1
  i.gem2 = gem2
  i.gem3 = gem3
  i.gem4 = gem4
  i.suffix = suffix
  i.unique = unique
  -- i.linklvl = linklvl
  i.linklvl = 100
  i.upgrade = upgrade
  i.mystery = mystery
  i.modifiercount = modifiercount
  for m = 1, modifiercount do
    i["modifier"..tostring(m)], modifiers = modifiers:match(":(%d+)(.*)")
  end
  i.name = name
  
  return true
end

function FantasyLoot:GetItem()
  if self.item.id then
    return packageItemLink(self.item)
  else
    return nil
  end
end

function FantasyLoot:ClearItem()
  self.item = {
    color = nil,
    id = nil,
    enchant = nil,
    gem1 = nil,
    gem2 = nil,
    gem3 = nil,
    gem4 = nil,
    suffix = nil,
    unique = nil,
    linklvl = nil,
    mystery = nil,
    upgrade = nil,
    name = nil,
  }
end

function FantasyLoot:GenerateGemTable()
  -- Update as nececcary for patch changes
  local GEM_START_ID = 5339
  local GEM_END_ID = 5351
  ----------------------------------------
  if not FantasyLootFrame then
    return
  end
  local gemToStatsByType = {
    Uncommon = {},
    Rare = {},
  }
  local jcidToType = function(id)
    if tonumber(id) < 49 then
      return "Uncommon"
    end
    return "Rare"
  end
  local texture,stats
  local testTemplate = "|cffff8000|Hitem:80211:0:%d:0:0:0:0:0:100:0:0:0|h[Enchanting Test Sword]|h|r"
  local count = 0
  
  FantasyLootFrameStatusFrame.StatusText:SetText(L["Scanning..."])
  
  for gem = GEM_START_ID, GEM_END_ID do
    FantasyLootScanningTooltip:ClearLines()
    FantasyLootScanningTooltip:SetHyperlink(format(testTemplate,gem))
    if FantasyLootScanningTooltipTexture1:IsShown() then
      texture = ((FantasyLootScanningTooltipTexture1:GetTexture() or ""):lower():match("interface\\icons\\(.+)") or "")
      if texture then
        jcid = texture:match("inv_jewelcrafting_(%d+)")
        if jcid then
          stats = select(2,FantasyLootScanningTooltipTexture1:GetPoint()):GetText():match("|c%x+(%+.+)|r")
          gemToStatsByType[jcidToType(jcid)][gem] = {text = stats, texture = texture}
          count = count + 1
        end
      end
    end
  end
  self.db.gemsKnown = count
  self.db.gemToStatsByType = gemToStatsByType
  
  FantasyLootFrameStatusFrame.StatusText:SetText(count..L[" gems discovered."])
end

function FantasyLoot:GetEnchantName(enchantID)
  local testTemplate = "|cffff8000|Hitem:80211:%d:0:0:0:0:0:0:100:0:0:0|h[Enchanting Test Sword]|h|r"
  FantasyLootScanningTooltip:ClearLines()
  FantasyLootScanningTooltip:SetHyperlink(format(testTemplate,enchantID))
  return (FantasyLootScanningTooltipTextLeft7:GetText() or ""):match(ENCHANTED_TOOLTIP_LINE:format("(.+)")) or ""
end

function FantasyLoot:GetItemUpgrade()
  local upgradeID = tonumber(self.item.upgrade)
  if upgradeID == 0 then
    return nil
  end
  for _, k in ipairs(upgradeByType) do
    local v = upgradeByType[k]
    if v.start ~= nil then
      if upgradeID >= v.start and upgradeID <= v.start + v.levels then
        return k
      end
    else
      for _, w in ipairs(v) do
        if upgradeID >= w.start and upgradeID <= w.start + w.levels then
          return k
        end
      end
    end
  end
  return nil
end

function FantasyLoot:GetItemSocketInfo(socket)
  local gem = tonumber(self.item["gem"..socket])
  local v
  for _,k in ipairs(gemIndex) do
    v = self.db.gemToStatsByType[k]
    if v[gem] then
      return v[gem].text, "Interface\\Icons\\"..v[gem].texture
    end
  end
  return nil
end

function FantasyLoot:DeltaApply(key, value)
  if key == "upgradeT" then
    if value == 0 then
      self.item.upgrade = 0
    end
  elseif key == "upgradeID" then
    self.item.upgrade = value
  elseif key == "enchantID" then
    self.item.enchant = value
  elseif key == "gem1" then
    self.item.gem1 = value
  elseif key == "gem2" then
    self.item.gem2 = value
  elseif key == "gem3" then
    self.item.gem3 = value
  elseif key == "gem4" then
    self.item.gem4 = value
  end
end